@@ -1,39 +1,19 @@
 # Convenience functions for working with spatial objects and leaflet maps
+getCallMethods = function(map) {
+  sapply(map$x$calls, "[[", "method")
+}
+
 
 getLayerControlEntriesFromMap <- function(map) {
-
-#   seq_along(map$x$calls)[sapply(map$x$calls,
-#                                 FUN = function(X) "addLayersControl" %in% X)]
-  tst <- which(sapply(map$x$calls, function(i) {
-    i$method == "addLayersControl"
-  }))
-  return(tst)
-
+  grep("addLayersControl", getCallMethods(map), fixed = TRUE, useBytes = TRUE)
 }
 
 
 getCallEntryFromMap <- function(map, call) {
-
-  #   seq_along(map$x$calls)[sapply(map$x$calls,
-  #                                 FUN = function(X) "addLayersControl" %in% X)]
-  tst <- which(sapply(map$x$calls, function(i) {
-    i$method == call
-  }))
-  return(tst)
-
+  grep(call, getCallMethods(map), fixed = TRUE, useBytes = TRUE)
 }
 
 
-# Get layer names of leaflet map ------------------------------------------
-
-getLayerNamesFromMap <- function(map) {
-
-  len <- getLayerControlEntriesFromMap(map)
-  len <- len[length(len)]
-  if (length(len) != 0) map$x$calls[[len]]$args[[2]] else NULL
-
-}
-
 
 # Query leaflet map for position of 'addProviderTiles' entry --------------
 
@@ -41,10 +21,11 @@ getProviderTileEntriesFromMap <- function(map) {
 
 #   seq_along(map$x$calls)[sapply(map$x$calls,
 #                                 FUN = function(X) "addProviderTiles" %in% X)]
-  tst <- which(sapply(map$x$calls, function(i) {
-    i$method == "addProviderTiles"
-  }))
-  return(tst)
+  # tst <- which(sapply(map$x$calls, function(i) {
+  #   i$method == "addProviderTiles"
+  # }))
+  grep("addProviderTiles", getCallMethods(map), fixed = TRUE, useBytes = TRUE)
+  # return(tst)
 
 }
 
@@ -61,13 +42,13 @@ getProviderTileNamesFromMap <- function(map) {
 
 # Update layer names of leaflet map ---------------------------------------
 
-updateLayerControlNames <- function(map1, map2) {
-  len <- getLayerControlEntriesFromMap(map1)
-  len <- len[length(len)]
-  map1$x$calls[[len]]$args[[2]] <- c(getLayerNamesFromMap(map1),
-                                     getLayerNamesFromMap(map2))
-  return(map1)
-}
+# updateLayerControlNames <- function(map1, map2) {
+#   len <- getLayerControlEntriesFromMap(map1)
+#   len <- len[length(len)]
+#   map1$x$calls[[len]]$args[[2]] <- c(getLayerNamesFromMap(map1),
+#                                      getLayerNamesFromMap(map2))
+#   return(map1)
+# }
 
 # Identify layers to be hidden from initial map rendering -----------------
 
@@ -89,32 +70,54 @@ getMapCalls <- function(map) {
 
 
 
+# Get layer names of leaflet map ------------------------------------------
+
+getLayerNamesFromMap <- function(map) {
+
+  len <- getLayerControlEntriesFromMap(map)
+  len <- len[length(len)]
+  if (length(len) != 0) map$x$calls[[len]]$args[[2]] else NULL
+
+}
+
+
+
 # Append calls to a map ---------------------------------------------------
 
 appendMapCallEntries <- function(map1, map2) {
+  ## calls
+  m1_calls = map1$x$calls
+  m2_calls = map2$x$calls
+
   ## base map controls
   ctrls1 <- getLayerControlEntriesFromMap(map1)
   ctrls2 <- getLayerControlEntriesFromMap(map2)
-  bmaps1 <- map1$x$calls[[ctrls1[1]]]$args[[1]]
-  bmaps2 <- map2$x$calls[[ctrls2[1]]]$args[[1]]
+  bmaps1 <- m1_calls[[ctrls1[1]]]$args[[1]]
+  bmaps2 <- m2_calls[[ctrls2[1]]]$args[[1]]
   bmaps <- c(bmaps1, bmaps2)[!duplicated(c(bmaps1, bmaps2))]
 
   ## layer controls
-  lyrs1 <- getLayerNamesFromMap(map1)
-  lyrs2 <- getLayerNamesFromMap(map2)
+  len1 <- ctrls1[length(ctrls1)]
+  lyrs1 = if (length(len1) != 0) m1_calls[[len1]]$args[[2]] else NULL
+  len2 <- ctrls2[length(ctrls2)]
+  lyrs2 = if (length(len2) != 0) m2_calls[[len2]]$args[[2]] else NULL
+  # lyrs1 <- getLayerNamesFromMap(map1)
+  # lyrs2 <- getLayerNamesFromMap(map2)
   lyrs <- c(lyrs1, lyrs2)
-  dup <- duplicated(lyrs)
-  lyrs[dup] <- paste0(lyrs[dup], ".2")
+  # dup <- duplicated(lyrs)
+  # lyrs[dup] <- sapply(seq(lyrs[dup]), function(i) paste0(lyrs[dup][[i]], ".", as.character(i + 1)))
 
   ## merge
-  mpcalls <- append(map1$x$calls, map2$x$calls)
+  mpcalls <- append(m1_calls, m2_calls)
   mpcalls <- mpcalls[!duplicated(mpcalls)]
   mpcalls[[ctrls1[1]]]$args[[1]] <- bmaps
   mpcalls[[ctrls1[1]]]$args[[2]] <- lyrs
 
-  ind <- which(sapply(mpcalls, function(i) {
-    i$method == "addLayersControl"
-  }))
+  # ind <- which(sapply(mpcalls, function(i) {
+  #   i$method == "addLayersControl"
+  # }))
+
+  ind =  grep("addLayersControl", sapply(mpcalls, "[[", "method"), fixed = TRUE, useBytes = TRUE)
 
 #   ind <- seq_along(mpcalls)[sapply(mpcalls,
 #                                    FUN = function(X) {
@@ -136,12 +139,18 @@ appendMapCallEntries <- function(map1, map2) {
 # Remove duuplicated map calls --------------------------------------------
 
 removeDuplicatedMapCalls <- function(map) {
-  ind <- anyDuplicated(map$x$calls)
-  for (i in ind) map$x$calls[[ind]] <- NULL
+  ind <- duplicated(getCallMethods(map))
+  if (any(ind)) map$x$calls[ind] <- NULL
   return(map)
 }
 
+# Remove duuplicated map dependencies -------------------------------------
 
+removeDuplicatedMapDependencies <- function(map) {
+  ind <- duplicated(map$dependencies)
+  if (any(ind)) map$dependencies[ind] <- NULL
+  return(map)
+}
 
 
 
@@ -189,12 +198,14 @@ initBaseMaps <- function(map.types) {
 
 # Initialise mapView map --------------------------------------------------
 
-initMap <- function(map, map.types, proj4str, native.crs = FALSE) {
+initMap <- function(map = NULL,
+                    map.types = NULL,
+                    proj4str,
+                    native.crs = FALSE) {
 
-  if (missing(map.types)) map.types <- mapviewGetOption("basemaps")
+  # if (missing(map.types)) map.types <- mapviewGetOption("basemaps")
 
-  if (missing(map) & missing(map.types)) {
-    map <- NULL
+  if (is.null(map) & is.null(map.types)) {
     map.types <- mapviewGetOption("basemaps")
   }
 
@@ -384,10 +395,12 @@ scaleLinesCoordinates <- function(x) {
 
 mapViewLayersControl <- function(map, map.types, names, native.crs = FALSE) {
 
-  if (!length(getLayerControlEntriesFromMap(map))) {
+  ind = getCallEntryFromMap(map, call = "addLayersControl")
+
+  if (!length(ind)) {
     bgm <- map.types
   } else {
-    bgm <- map$x$calls[[getLayerControlEntriesFromMap(map)[1]]]$args[[1]]
+    bgm <- map$x$calls[[ind[1]]]$args[[1]]
   }
 
   if (!native.crs) {
@@ -411,6 +424,18 @@ mapViewLayersControl <- function(map, map.types, names, native.crs = FALSE) {
 }
 
 
+# Update leaflet layers control button ------------------------------------
+updateOverlayGroups = function(map, group) {
+  if (inherits(map, "mapview")) map = mapview2leaflet(map)
+  ind = getLayerControlEntriesFromMap(map)
+  if (length(ind > 0)) {
+    map$x$calls[ind][[1]]$args[[2]] =
+      c(map$x$calls[ind][[1]]$args[[2]], group)
+  }
+  return(map)
+}
+
+
 # Create layer name for grouping in map -----------------------------------
 
 # layerName <- function() {
